const bcrypt = require("bcrypt");

const { User } = require("../model/usr");
const { Otp } = require("../model/otp");

const signUp = async (req, res) => {
  try {
    const input = req.body.number;

    // validating user
    let user = await User.findOne({ number: input });
    if (user) return res.status(409).json({ info: "User already registered" });

    // save user
    const usr = new User({ number: input });
    await usr.save();

    // generating otp
    let random = Math.floor(Math.random() * 90000) + 10000;
    console.log(random);

    // saving otp in db
    const otp = new Otp({ number: input, otp: random });
    const salt = await bcrypt.genSalt(10);
    otp.otp = await bcrypt.hash(otp.otp, salt);
    const result = await otp.save();
    console.log("saved", result);

    return res.status(201).json({ data: random, info: "Otp generated" });
  } catch (err) {
    return res.status(400).json({ info: err });
  }
};

const verifyOtp = async (req, res) => {
  try {
    const input = req.body.number;
    const pwd = req.body.otp;

    if (pwd.length === 0)
      return res.status(400).json({ info: "Otp cannot be blank" });

    // verifying user
    const otp = await Otp.find({ number: input });
    if (!otp) return res.status(404).json({ info: "otp not found" });

    // comparing otp
    const details = otp[otp.length - 1];
    const valid = await bcrypt.compare(pwd, details.otp);

    if (input === details.number && valid) {
      // generating token
      const user = new User({ number: input });
      const token = user.generateTkn();
      // todo - save jwt tkn in db for comparison later
      // access_token for further usage
      return res
        .status(201)
        .json({ access_token: token, info: "Token generated" });
    } else {
      return res.status(400).json({ info: "Bad request" });
    }
  } catch (err) {
    return res.status(400).json({ info: err });
  }
};

module.exports = {
  signUp,
  verifyOtp
};
